;;;; cxterm.el
;;;; by Stephen G. Simpson <simpson@math.psu.edu>
;;;; last revised 22 Sept 1991

;; DESCRIPTION
;; This code enables Emacs to edit Chinese files within cxterm.

;; SOFTWARE REQUIREMENTS
;; It is assumed that your Emacs has the 8-bit ctl-arrow patch.

;; BUGS
;; This code is very primitive.  If you make any improvements, please
;; send them to Steve Simpson (simpson@math.psu.edu), Mark Leisher
;; (mleisher@nmsu.edu) and Yongguang Zhang (ygz@cs.purdue.edu).

;; HOW TO USE THIS FILE
;;
;; Your .emacs and .cshrc respectively can contain the following lines.
;;
;; (autoload 'global-cxterm-mode "cxterm"
;;    "Enable cxterm mode in current and all subsequently created buffers.
;; For editing Chinese files inside cxterm." t)
;;
;; alias cemacs 'emacs -nw \!* -f global-cxterm-mode'
;; alias cxemacs 'cxterm -e "emacs -nw \!* -f global-cxterm-mode" &'
;;
;; You can then invoke cemacs within cxterm to edit Chinese files,
;; or cxemacs to start a new cxterm and edit Chinese files
;; within it.

;; let the user choose whether to redisplay
;; after every cursor movement in cxterm mode
(defvar cxterm-auto-redisplay t
  "*If non nil, cxterm-mode cursor keys automatically redisplay the screen.
This eliminates some messy screens, but it also causes an annoying blink.
If you prefer the messy screens to the blinking, set this variable to nil.
You can always redisplay the screen yourself by hitting C-l.")
(defun cxterm-auto-redisplay ()
   (if cxterm-auto-redisplay (redraw-display) nil))

;; interactive commands for cxterm mode
(defun cxterm-forward-char (&optional arg)
  "cxterm-mode replacement for forward-char."
  (interactive "p")
  (while (> arg 0)
    (forward-char (if (> (char-after (point)) 128) 2 1))
    (setq arg (- arg 1)))
  (cxterm-auto-redisplay))
(defun cxterm-backward-char (&optional arg)
  "cxterm-mode replacement for backward-char."
  (interactive "p")
  (while (> arg 0)
    (backward-char (if (> (char-after (- (point) 1)) 128) 2 1))
    (setq arg (- arg 1)))
  (cxterm-auto-redisplay))
(defun cxterm-delete-char (&optional arg)
  "cxterm-mode replacement for delete-char."
  (interactive "p")
  (while (> arg 0)  
    (delete-char (if (> (char-after (point)) 128) 2 1))
    (setq arg (- arg 1)))
  (cxterm-auto-redisplay))
(defun cxterm-backward-delete-char-untabify (&optional arg)
  "cxterm-mode replacement for backward-delete-char-untabify."
  (interactive "p")
  (while (> arg 0)  
    (backward-delete-char-untabify 
      (if (> (char-after (- (point) 1)) 128) 2 1))
    (setq arg (- arg 1)))
  (cxterm-auto-redisplay))
(defun cxterm-previous-line (arg)
  "cxterm replacement for previous-line."
  (interactive "p")
  (previous-line arg)
  (cxterm-auto-redisplay))
(defun cxterm-next-line (arg)
  "cxterm replacement for next-line."
  (interactive "p")
  (next-line arg)
  (cxterm-auto-redisplay))
(defun cxterm-beginning-of-line ()
  "cxterm replacement for beginning-of-line."
  (interactive)
  (beginning-of-line)
  (cxterm-auto-redisplay))
(defun cxterm-end-of-line ()
  "cxterm replacement for end-of-line."
  (interactive)
  (end-of-line)
  (cxterm-auto-redisplay))

;; set up keymap for 8-bit characters
(setq insert-meta-map (make-keymap))
(setq num 0)
(while (< num 128)
  (define-key insert-meta-map (char-to-string num) 
    (concat "\C-Q" (char-to-string (+ 128 num))))
  (setq num (+ num 1)))

;; create cxterm mode, a minor mode
(setq minor-mode-alist 
  (append minor-mode-alist '((cxterm-mode " CXTERM"))))
(defvar cxterm-mode nil
   "Used internally by the cxterm-mode function.")
(defun cxterm-mode (&optional off)
    "Enable cxterm mode in the current buffer.
For editing Chinese files inside cxterm.
With ARG, disable cxterm mode in the current buffer."
    (interactive "P")
    (make-local-variable 'cxterm-mode)
    (if off
	(progn	
	  (setq cxterm-mode nil)
	  (setq ctl-arrow nil)
	  (local-set-key "\C-f" 'forward-char)
	  (local-set-key "\eOC" 'forward-char)
	  (local-set-key "\e[C" 'forward-char)
	  (local-set-key "\C-b" 'backward-char)
	  (local-set-key "\eOD" 'backward-char)
	  (local-set-key "\e[D" 'backward-char)
	  (local-set-key "\C-d" 'delete-char)
	  (local-set-key "\C-h" 'delete-char)
	  (local-set-key "\eOn" 'delete-char)
	  (local-set-key "\177" 'backward-delete-char-untabify)
	  (local-set-key "\C-p" 'previous-line)
	  (local-set-key "\eOA" 'previous-line)
	  (local-set-key "\e[A" 'previous-line)
	  (local-set-key "\C-n" 'next-line)
	  (local-set-key "\eOB" 'next-line)
	  (local-set-key "\e[B" 'next-line)
          (local-set-key "\C-a" 'beginning-of-line)
          (local-set-key "\eOP" 'beginning-of-line)
          (local-set-key "\C-e" 'end-of-line)
          (local-set-key "\eOQ" 'end-of-line)
	  (setq meta-flag t))
        (setq cxterm-mode t)
 	(setq ctl-arrow 2)
	(local-set-key "\C-f" 'cxterm-forward-char)
	(local-set-key "\eOC" 'cxterm-forward-char)
	(local-set-key "\e[C" 'cxterm-forward-char)
	(local-set-key "\C-b" 'cxterm-backward-char)
	(local-set-key "\eOD" 'cxterm-backward-char)
	(local-set-key "\e[D" 'cxterm-backward-char)
	(local-set-key "\C-d" 'cxterm-delete-char)
	(local-set-key "\C-h" 'cxterm-delete-char)
	(local-set-key "\eOn" 'cxterm-delete-char)
	(local-set-key "\177" 'cxterm-backward-delete-char-untabify)
	(local-set-key "\C-p" 'cxterm-previous-line)
	(local-set-key "\eOA" 'cxterm-previous-line)
	(local-set-key "\e[A" 'cxterm-previous-line)
	(local-set-key "\C-n" 'cxterm-next-line)
	(local-set-key "\eOB" 'cxterm-next-line)
	(local-set-key "\e[B" 'cxterm-next-line)
	(local-set-key "\C-a" 'cxterm-beginning-of-line)
	(local-set-key "\eOP" 'cxterm-beginning-of-line)
	(local-set-key "\C-e" 'cxterm-end-of-line)
	(local-set-key "\eOQ" 'cxterm-end-of-line)
	(local-set-key (char-to-string 28) insert-meta-map)
	;; Unfortunately, the next variable is global.
	(setq meta-prefix-char 28)
	(setq meta-flag t)))

;; global cxterm mode -- start all buffers in cxterm mode
(defun global-cxterm-mode ()
   "Enable cxterm mode in current and all subsequently created buffers.
For editing Chinese files inside cxterm."
    (interactive)
    (cxterm-mode)
    (setq find-file-hooks 
       (cons 'cxterm-mode find-file-hooks))
    (message "All files will be found in cxterm mode."))

;; provide this package
(provide 'cxterm)
